//=============================================================================
// TcpLink: An Internet TCP/IP connection.
//=============================================================================
class TcpLink expands InternetLink
	intrinsic
	transient;

//-----------------------------------------------------------------------------
// Variables.

// LinkState is only valid for TcpLink at this time.
var enum ELinkState
{
	STATE_Initialized,	// Sockets is initialized
	STATE_Ready,		// Port bound, ready for activity
	STATE_Listening,	// Listening for connections
	STATE_Connecting,	// Attempting to connect
	STATE_Connected,	// Open and connected
} LinkState;

var IpAddr	  RemoteAddr;	// Contains address of peer connected to from a Listen()
var const int DataPending;	// Indicates whether data is available to be read.

//-----------------------------------------------------------------------------
// Intrinsics.

// BindPort: Binds a free port or optional port specified in argument one.
intrinsic function bool BindPort( optional int Port, optional bool bUseNextAvailable );

// Listen: Listen for connections.  Can handle up to 5 simultaneous connections.
// Returns false if failed to place socket in listen mode.
intrinsic function bool Listen();

// Open: Open a connection to a foreign host.
intrinsic function bool Open( IpAddr Addr );

// Close: Closes the current connection.   
intrinsic function bool Close();

// IsConnected: Returns true if connected.
intrinsic function bool IsConnected();

// IsDataPending: Returns true if data is pending on the socket.
intrinsic function bool IsDataPending();

// SendText: Sends text string. 
// Appends a cr/lf if LinkMode=MODE_Line.  Returns number of bytes sent.
intrinsic function int SendText( coerce string[240] Str );

// SendBinary: Send data as a byte array.
// NOTE: CURRENTLY BROKEN DUE TO LIMITATIONS IN UNREALSCRIPT
intrinsic function int SendBinary( byte B[240] );

// ReadText: Reads text string.
// Returns number of bytes read.  
intrinsic function int ReadText( out string[240] Str );

// ReadBinary: Read data as a byte array.
// NOTE: CURRENTLY BROKEN DUE TO LIMITATIONS IN UNREALSCRIPT
intrinsic function int ReadBinary( out byte B[240] );

//-----------------------------------------------------------------------------
// Events.

// Accepted: Called during STATE_Listening when a new connection is accepted.
event Accepted();

// Opened: Called when socket successfully connects.
event Opened();

// Closed: Called when Close() completes or the connection is dropped.
event Closed();

defaultproperties
{
     bAlwaysTick=True
}
