//=============================================================================
// TeleporterPinger.
//=============================================================================
class TeleporterPinger expands UdpLink;

var bool		Initialized;

var IpAddr		RemoteIP;			// IP Address and Port of remote server.
var string[64]	RemoteLevel;		// Name of remote level.
var string[64]	RemotePortal;		// Name of remote teleporter.

var float		TimeElapsed;

// Maintain two lists of up to 8 players that need to be told of their Pings;
struct ActorInfo
{
	var string[64]  Name;
	var float		Time;
	var int			Ping;
};
var ActorInfo	Info[8];

var int			UpdateIndex;

// Result Information
var string[240] HostName, MapName, GameType, NumPlayers, MaxPlayers;

function InitializeInfo( string[64] URL )
{
	local string[64] RemoteAddress;
	local bool Result;
	local int i;

	if ( BindPort() )
	{
		log( "TeleporterPinger successfully bound." );
	}
	else
	{
		log( "TeleporterPinger Error: Could not bind port." );
	}

	Log( "TeleporterPinger: Remote URL is "$URL );

	SetTimer(30, true);

	Result = ParseURL( URL, RemoteAddress, RemoteIP.Port,
					   RemoteLevel, RemotePortal );

	RemoteIP.Port = 8777;

	if ( Result )
		Resolve( RemoteAddress );
	else
		Log( "TeleporterPinger Error: Failed to parse URL." );
}

function ReportStats( Actor Other )
{
	if ( (!Other.IsA('PlayerPawn')) || (!Initialized))
		return;

	Info[UpdateIndex].Name = string(PlayerPawn(Other).Name);
	Info[UpdateIndex].Time = TimeElapsed;

	SendText( RemoteIP, "\\echo\\"$PlayerPawn(Other).Name );
	UpdateIndex++;
}

function Resolved(IpAddr Addr)
{
	RemoteIP.Addr = Addr.Addr;

	Log( "TeleportPinger: Teleporter resolved." );
	SendText( RemoteIP, "\\info\\" );

	Initialized = true;
}

function ReceivedText( IpAddr Addr, string[240] Text )
{
	local int QueryNum, PacketNum;
	local string[240] Query;
	local bool QueryRemaining;

	Query = Text;
	if (Query == "")	// If the string is empty, don't parse it
		QueryRemaining = false;
	else
		QueryRemaining = true;
	
	while (QueryRemaining) {
		Query = ParseQuery(Addr, Query, QueryNum, PacketNum);
		if (Query == "")
			QueryRemaining = false;
		else
			QueryRemaining = true;
	}
}

function string[240] ParseQuery(IpAddr Addr, string[240] Query, int QueryNum, out int PacketNum)
{
	local string[240] EchoName, QueryType, QueryValue, QueryRest;
	local bool Result;
	local string[32] FinalPacket;
	local float StartTime;
	local PlayerPawn P;
	local int i, Ping;

	Result = ParseNextQuery(Query, QueryType, QueryValue, QueryRest, FinalPacket);
	
	if (!Result)
		return "";
	switch ( QueryType )
	{
		case "hostname":
			HostName = QueryValue;
			break;
		case "mapname":
			MapName = QueryValue;
			break;
		case "gametype":
			GameType = QueryValue;
			break;
		case "numplayers":
			NumPlayers = QueryValue;
			break;
		case "maxplayers":
			MaxPlayers = QueryValue;
			break;
		case "echo":
			foreach AllActors(class'PlayerPawn', P)
				if (QueryValue == string(P.Name))
				{
					for (i = 0; i < 8; i++)
						if (QueryValue == Info[i].Name)
						{
							StartTime = Info[i].Time;
							Info[i].Ping = (TimeElapsed - StartTime) * 1000;
						}
					P.ClientMessage("Remote Desination Information");
					P.ClientMessage("Server Name: "$HostName);
					P.ClientMessage("Level: "$MapName);
					P.ClientMessage("Players: "$NumPlayers$"/"$MaxPlayers);
					P.ClientMessage("GameType: "$GameType);
					P.ClientMessage("Ping: "$Ping$"ms");
				}
			break;
	}

	return QueryRest;
}

function Timer()
{
	if (Initialized)
		SendText( RemoteIP, "\\info\\" );
}

function Tick(float Delta)
{
	TimeElapsed += Delta;
}

defaultproperties
{
}
